require("babel-polyfill");

import fs from 'fs';
import os from 'os';
import path from 'path';
import process from 'process';

import superagent from 'superagent';
import mkdirp from 'mkdirp';
import mime from 'mime';
import {promisify} from 'bluebird';
import margarita from 'margarita';
import inquirer from 'inquirer';

import jwt from 'jsonwebtoken';
import eachLimit from 'async/eachLimit';
import mapLimit from 'async/mapLimit';
import asyncify from 'async/asyncify';
import _ from 'underscore';
import chalk from 'chalk';
import ProgressBar from 'progress';

import traverse from './traverse';

const CONFIG_PATH = path.resolve(os.homedir(), '.config/puche');

// const HOST = "http://api.staging.puche.io";
const HOST = "http://api.puche.io";

var config = {
    apiKey: "AIzaSyBtdHYms6uGue_ZmwlNtGCMol2SThyaZBs",
    authDomain: "puche-io.firebaseapp.com",
};

// var config = {
//     apiKey: "AIzaSyDQRgfrzWK2GYMDOjMkJ9xaXoi8SlWrcUM",
//     authDomain: "staging-puche-io.firebaseapp.com"
// };

const upload = async function({siteId}) {

    let jwtToken;
    try {
        let tokenFile = await promisify(fs.readFile)(path.join(CONFIG_PATH, 'token'));
        jwtToken = JSON.parse(tokenFile);
    } catch(e) {
        jwtToken = await auth();
    }

    let token = jwtToken.idToken;
    let tokenObject = jwt.decode(token);

    if (tokenObject.exp < new Date().getTime()/1000) {
        token = (await refreshToken(jwtToken)).id_token;
    }

    let files = await traverse('.');

    console.log(`Uploading ${files.length} files to ${chalk.keyword('orange')(siteId)}`);

    let fileGroups = _.map(_.groupBy(files, (file, index) => Math.floor(index/50)), (fileList, index) => ({fileList, index}));

    var bar = new ProgressBar('Progress [:bar] :percent', {
        complete: '=',
        incomplete: ' ',
        width: 40,
        total: files.length * 2
    });
    bar.tick(0);

    let fileUploads = await promisify(mapLimit)(fileGroups, 1, asyncify(async function ({fileList, index}) {

        let response = await superagent.post(`${HOST}/api/v1/signUploads`)
            .set("Authorization", token)
            .send({
                site: siteId,
                uploads: fileList.map((file) => ({
                    contentType: mime.lookup(file),
                    path: path.relative(process.cwd(), file)
                }))
            });

        bar.tick(response.body.signedUploads.length);

        return _.zip(fileList, response.body.signedUploads).map(([file, url]) => ({file, url}));
    }));

    await promisify(eachLimit)(_.flatten(fileUploads), 20, asyncify(async function (fileUpload) {

        let data = await promisify(fs.readFile)(fileUpload.file);

        await superagent.put(fileUpload.url)
            .set("Content-Type", mime.lookup(fileUpload.file))
            .set("Cache-Control", "public, max-age=60")
            .set("x-goog-acl", "public-read")
            .send(data);

        bar.tick(1);

    }));

    let configFile = JSON.parse(await promisify(fs.readFile)(path.join(process.cwd(), 'puche.json')));

    console.log(configFile);

    let response = await superagent.put(`${HOST}/api/v1/accounts/m2y71RMM96A/sites/${siteId}/config`)
        .set("Authorization", token)
        .send(configFile);

    console.log("Upload Complete.")
};

const signIn = async function(email, password) {
    let response = await superagent
        .post(`https://www.googleapis.com/identitytoolkit/v3/relyingparty/verifyPassword?key=${config['apiKey']}`)
        .send({
            email,
            password,
            returnSecureToken: true
        });
    return response.body;
};

const refreshToken = async function(jwtToken) {
    let response = await superagent
        .post(`https://securetoken.googleapis.com/v1/token?key=${config['apiKey']}`)
        .send({
            grant_type: 'refresh_token',
            refreshToken: jwtToken.refreshToken
        });
    return response.body;
};


const auth = async function() {
    let results = await inquirer.prompt([
        {type:'input', message: "Username", name: "username"},
        {type:'password', message: "Password", name: "password"},
    ]);

    let tokenResponse = await signIn(results['username'], results['password']);
    await promisify(mkdirp)(CONFIG_PATH);
    await promisify(fs.writeFile)(path.join(CONFIG_PATH, 'token'), JSON.stringify(tokenResponse));
    return tokenResponse;
};

const writeToken = async function(tokenObj) {
    await promisify(mkdirp)(CONFIG_PATH);
    return promisify(fs.writeFile)(path.join(CONFIG_PATH, 'token'), JSON.stringify(tokenResponse.body));
};

let program = margarita();

program
    .command('auth')
    .action((args) => {
        auth().catch(e => console.error("Error:", e));
    });

program
    .command('upload')
    .option('s', 'site')
    .action((args) => {
        upload({siteId: args.site}).catch(e => console.error("Error:", e));
    });

program.parse(process.argv).catch(e => console.error("NoCommand Error:", e));
